"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var SelfClosingHTMLElements_1 = __importDefault(require("../html-config/SelfClosingHTMLElements"));
var UnnestableHTMLElements_1 = __importDefault(require("../html-config/UnnestableHTMLElements"));
var he_1 = require("he");
var NamespaceURI_1 = __importDefault(require("../html-config/NamespaceURI"));
var MARKUP_REGEXP = /<(\/?)([a-z][-.0-9_a-z]*)\s*([^>]*?)(\/?)>/gi;
var ATTRIBUTE_REGEXP = /([^\s=]+)(?:\s*=\s*(?:"([^"]*)"|'([^']*)'|(\S+)))/gi;
var XMLNS_ATTRIBUTE_REGEXP = /xmlns[ ]*=[ ]*"([^"]+)"/;
/**
 * HTML parser.
 */
var HTMLParser = /** @class */ (function () {
    function HTMLParser() {
    }
    /**
     * Parses HTML and returns a root element.
     *
     * @param  {Document} document Document.
     * @param  data HTML data.
     * @return Root element.
     */
    HTMLParser.parse = function (document, data) {
        var root = document.createElement('root');
        var stack = [root];
        var markupRegexp = new RegExp(MARKUP_REGEXP, 'gi');
        var parent = root;
        var parentUnnestableTagName = null;
        var lastTextIndex = 0;
        var match;
        while ((match = markupRegexp.exec(data))) {
            var tagName = match[2].toLowerCase();
            var isStartTag = !match[1];
            if (parent && match.index !== lastTextIndex) {
                var text = data.substring(lastTextIndex, match.index);
                this.appendTextAndCommentNodes(document, parent, text);
            }
            if (isStartTag) {
                var newElement = document.createElement(tagName);
                var xmlnsAttribute = this.getXmlnsAttribute(match[3]);
                // The HTML engine can guess that the namespace is SVG for SVG tags
                // even if "xmlns" is not set if the parent namespace is HTML.
                if (tagName === 'svg' && parent._namespaceURI === NamespaceURI_1.default.html) {
                    newElement._namespaceURI = xmlnsAttribute || NamespaceURI_1.default.svg;
                }
                else {
                    newElement._namespaceURI = xmlnsAttribute || parent._namespaceURI;
                }
                this.setAttributes(newElement, newElement._namespaceURI, match[3]);
                if (!SelfClosingHTMLElements_1.default.includes(tagName)) {
                    // Some elements are not allowed to be nested (e.g. "<a><a></a></a>" is not allowed.).
                    // Therefore we will auto-close the tag.
                    if (parentUnnestableTagName === tagName) {
                        stack.pop();
                        parent = parent.parentNode || root;
                    }
                    parent = parent.appendChild(newElement);
                    parentUnnestableTagName = this.getUnnestableTagName(parent);
                    stack.push(parent);
                }
                else {
                    parent.appendChild(newElement);
                }
                lastTextIndex = markupRegexp.lastIndex;
            }
            else {
                stack.pop();
                parent = stack[stack.length - 1] || root;
                parentUnnestableTagName = this.getUnnestableTagName(parent);
                lastTextIndex = markupRegexp.lastIndex;
            }
        }
        // Text after last element
        if ((!match && data.length > 0) || (match && lastTextIndex !== match.index)) {
            var text = data.substring(lastTextIndex);
            this.appendTextAndCommentNodes(document, root, text);
        }
        return root;
    };
    /**
     * Returns a tag name if element is unnestable.
     *
     * @param element Element.
     * @return Tag name if element is unnestable.
     */
    HTMLParser.getUnnestableTagName = function (element) {
        var tagName = element.tagName.toLowerCase();
        return tagName && UnnestableHTMLElements_1.default.includes(tagName) ? tagName : null;
    };
    /**
     * Appends text and comment nodes.
     *
     * @param  {Document} document Document.
     * @param  node Node.
     * @param  text Text to search in.
     */
    HTMLParser.appendTextAndCommentNodes = function (document, node, text) {
        for (var _i = 0, _a = this.getTextAndCommentNodes(document, text); _i < _a.length; _i++) {
            var innerNode = _a[_i];
            node.appendChild(innerNode);
        }
    };
    /**
     * Returns text and comment nodes from a text.
     *
     * @param  {Document} document Document.
     * @param  text Text to search in.
     * @returns Nodes.
     */
    HTMLParser.getTextAndCommentNodes = function (document, text) {
        var nodes = [];
        var commentRegexp = /<!--(.*?)-->/gms;
        var lastIndex = 0;
        var match;
        while ((match = commentRegexp.exec(text))) {
            if (match.index > 0) {
                var textNode = document.createTextNode(text.substring(lastIndex, match.index));
                nodes.push(textNode);
            }
            var commentNode = document.createComment(match[1]);
            nodes.push(commentNode);
            lastIndex = match.index + match[0].length;
        }
        if (lastIndex < text.length) {
            var textNode = document.createTextNode(text.substring(lastIndex));
            nodes.push(textNode);
        }
        return nodes;
    };
    /**
     * Returns XMLNS attribute.
     *
     * @param attributesString Raw attributes.
     */
    HTMLParser.getXmlnsAttribute = function (attributesString) {
        var match = attributesString.match(XMLNS_ATTRIBUTE_REGEXP);
        return match ? match[1] : null;
    };
    /**
     * Sets raw attributes.
     *
     * @param element Element.
     * @param namespaceURI Namespace URI.
     * @param attributesString Raw attributes.
     */
    HTMLParser.setAttributes = function (element, namespaceURI, attributesString) {
        var attributes = attributesString.trim();
        if (attributes) {
            var regExp = new RegExp(ATTRIBUTE_REGEXP, 'gi');
            var match = void 0;
            // Attributes with value
            while ((match = regExp.exec(attributes))) {
                if (match[1]) {
                    element.setAttributeNS(null, this._getAttributeName(namespaceURI, match[1]), he_1.decode(match[2] || match[3] || match[4] || ''));
                }
            }
            // Attributes with no value
            for (var _i = 0, _a = attributes
                .replace(ATTRIBUTE_REGEXP, '')
                .trim()
                .split(' '); _i < _a.length; _i++) {
                var name = _a[_i];
                if (name) {
                    element.setAttributeNS(null, this._getAttributeName(namespaceURI, name), '');
                }
            }
        }
    };
    /**
     * Returns attribute name.
     *
     * @param namespaceURI Namespace URI.
     * @param name Name.
     * @returns Attribute name based on namespace.
     */
    HTMLParser._getAttributeName = function (namespaceURI, name) {
        if (namespaceURI === NamespaceURI_1.default.svg) {
            return name;
        }
        return name.toLowerCase();
    };
    return HTMLParser;
}());
exports.default = HTMLParser;
//# sourceMappingURL=HTMLParser.js.map